#!/bin/bash
#(c) Tadeus Dobrovolskij, 2011
#Script to rar and par your usenet uploads
############################
#
# Variables
############################
# File name prefix
NAME_PREFIX=
# File name suffix
NAME_SUFFIX=
# Rar volume size in megabytes 
VOLUME_SIZE=50
# Par block size
BLOCK_SIZE=384000
# Par recovery %
PAR_RECOVERY=10
# Move new files to subdirectories or not?
DIRECTORIES=0
# Rar command
RARC=
# Par2 command
PAR2C=
# File which will be processed by script next
FILE_TO_PROCESS=
# FILENAME=prefix+(actual_file_name - extension)+suffix
FILENAME=
# Use rar names of the new type?
OLDRAR=0
# Rar compression level
COMPRESSION=0
# Uniform par file size
UNIFORM=0
# Limit size of par2 recovery files. Can't be used with UNIFORM=1
LIMIT=0
# Process files larger than minimum size
MINIMUMS=0
# Verbosity
VERBOSE=0

# Simple check to find out if you have installed needed tools. Tested in Ubuntu and Gentoo. You may need to change this part, if you have those tools in different place.
if [ ! -e /usr/bin/par2 ]; then
	echo "Error: par2 not found!"
	exit 1
fi
if [ ! -e /usr/bin/rar ] && [ ! -e /opt/rar/bin/rar ] ; then
	echo "Error: rar not found!"
	exit 1
fi

# Simple help. Maybe there is a better way to do this, but this should be fine for a time being.
show_help() {
	echo "Rar&Par script version 0.22. Copyright (C) 2011 Tadeus Dobrovolskij."
	echo -e "Comes with ABSOLUTELY NO WARRANTY.\n"
	echo "Script helps you prepare your files for Usenet. Each file in the current directory is archived with RAR, then par2 files are created."
	echo -e "Must have par2 and rar installed (obviously).\n"
	echo -e "Usage: $0 [options]\n"
	echo -e "Options:\n"
	echo -e "\t-b <size>\tPar block size. Default: \033[1m384000\033[0m"
	echo -e "\t-d\t\tCreates directory for each processed file's rar and par2 files."
	echo -e "\t-h\t\tDisplay this message. Same as '--help'."
	echo -e "\t-l\t\tLimit size of par2 recovery files (Don't use both -u and -l)."
	echo -e "\t-m <0..5>\tRar compression level (0-store...5-maximal). Default: \033[1m0\033[0m"
	echo -e "\t-min <size>\tProcess only the files larger than <size> megabytes."
	echo -e "\t-n\t\tuse old style rar file names."
	echo -e "\t-p <string>\tFile name prefix."
	echo -e "\t-r <%%>\t\tPar recovery per cent. Default: \033[1m10\033[0m"
	echo -e "\t-s <string>\tFile name suffix."
	echo -e "\t-u\t\tUniform par2 recovery file size."
	echo -e "\t-v <MB>\t\tRar volume size in megabytes. Default:  \033[1m50\033[0m"
	echo -e "\t-V\t\tBe verbose.\n"
}
# Options parser. Checks options beginning with a dash("-") symbol.
while [[ $1 = -* ]]; do
	case "$1" in
		-b)
		 BLOCK_SIZE="$2"
		 shift 2
		 ;;
		-d)
		 DIRECTORIES=1
		 shift
		 ;;
		-h|--help)
		 show_help
		 exit
		 ;;
		-l)
		 LIMIT=1
		 shift
		 ;;
		-m)
		 COMPRESSION="$2"
		 shift 2
		 ;;
		-min)
		 MINIMUMS="$2"
		 shift 2
		 ;;
		-n)
		 OLDRAR=1
		 shift
		 ;;
		-p)
		 NAME_PREFIX="$2"
		 shift 2
		 ;;
		-r)
		 PAR_RECOVERY="$2"
		 shift 2
		 ;;
		-s)
		 NAME_SUFFIX="$2"
		 shift 2
		 ;;
		-u)
		 UNIFORM=1
		 shift
		 ;;
		-v)
		 VOLUME_SIZE="$2"
		 shift 2
		 ;;
		-V)
		 VERBOSE=1
		 shift
		 ;;
		*)
		 echo "Error: Unknown option: $1" >&2
		 show_help
		 exit 1
		 ;;
	esac
done
# Setting up rar command to be exactly like we need it to be.
RARC="rar a -rr1 -ep -m${COMPRESSION} -v${VOLUME_SIZE}m -ed"
if (( $OLDRAR==1 )); then
	RARC="${RARC} -vn"
fi
# Same for par2 command.
PAR2C="par2 c -s${BLOCK_SIZE} -r${PAR_RECOVERY}"
if (( $UNIFORM==1 )); then
	PAR2C="${PAR2C} -u"
fi
if (( $UNIFORM==0 )) && (( $LIMIT==1)); then
	PAR2C="$PAR2C -l"
fi

# First, we look for all files in current directory according to our minimum size limit.
if (($(find . -maxdepth 1 -type f \( ! -iname ".*" \) -size +${MINIMUMS}M | wc -l) > 0)); then
	while read FILE_TO_PROCESS; do
		echo "Processing: $FILE_TO_PROCESS"
		FILENAME="$NAME_PREFIX""${FILE_TO_PROCESS%.*}""$NAME_SUFFIX"
		mkdir "${FILENAME}"
		# If verbosity option is set, then output all the stuff to stdo. 
		if (( $VERBOSE==1 )); then
			$RARC "${FILENAME}"/"${FILENAME}".rar "$FILE_TO_PROCESS"
			$PAR2C "${FILENAME}"/"${FILENAME}".par2 "${FILENAME}"/"${FILENAME}"*.r*
		else	
		# Otherwise just continue.
			$RARC "${FILENAME}"/"${FILENAME}".rar "$FILE_TO_PROCESS" > /dev/null 2>&1
			$PAR2C "${FILENAME}"/"${FILENAME}".par2 "${FILENAME}"/"${FILENAME}"*.r* > /dev/null 2>&1
		fi
		# Store rared files in separate directory?
		if (( $DIRECTORIES==0 )); then
			mv "${FILENAME}"/* .
			rmdir "${FILENAME}"/
		fi
	done < <(find . -maxdepth 1 -type f \( ! -iname ".*" \) -size +${MINIMUMS}M | sed -e 's/.\///g')
	else
		echo "Error: No files to process!"
		exit 1
fi
echo "Task completed."
